@@ -1,11 +1,7 @@
 #include <config.h>
 #include <string.h>
 #include <unistd.h>
-#include <glib/gthread.h>
-#include <glib/gqueue.h>
-#include <gtk/gtkmain.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-thumbnail.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnomevfs/gnome-vfs.h>
 #ifdef HAVE_EXIF
@@ -17,7 +13,6 @@
 #include "libeog-marshal.h"
 #include "eog-image.h"
 #include "eog-image-private.h"
-#include "eog-pixbuf-util.h"
 #include "eog-metadata-reader.h"
 #include "eog-image-save-info.h"
 #include "eog-info-view-file.h"
@@ -75,7 +70,6 @@ static void
 eog_image_dispose (GObject *object)
 {
 	EogImagePrivate *priv;
-	GList *it;
 
 	priv = EOG_IMAGE (object)->priv;
 
@@ -122,11 +116,8 @@ eog_image_dispose (GObject *object)
 	}
 
 	if (priv->undo_stack) {
-		for (it = priv->undo_stack; it != NULL; it = it->next){
-			g_object_unref (G_OBJECT (it->data));
-		}
-
-		g_list_free (priv->undo_stack);
+		g_slist_foreach (priv->undo_stack, (GFunc) g_object_unref, NULL);
+		g_slist_free (priv->undo_stack);
 		priv->undo_stack = NULL;
 	}
 }
@@ -486,107 +477,6 @@ eog_image_determine_file_bytes (EogImage *img, GError **error)
 	return bytes;
 }
 
-/* this function runs in it's own thread */
-static gboolean
-eog_image_load_exif_data_only (EogImage *img, EogJob *job, GError **error)
-{
-	EogImagePrivate *priv;
-	GnomeVFSHandle *handle;
-	guchar *buffer;
-	GnomeVFSFileSize bytes_read;
-	GnomeVFSFileSize bytes_read_total;
-	GnomeVFSResult result;
-	gboolean failed;
-	gboolean first_run = TRUE;
-	EogMetadataReader *md_reader = NULL;
-
-	g_assert (error == NULL || *error == NULL);
-
-	priv = img->priv;
-
-#ifdef DEBUG
-	g_print ("real image exif load %s\n", gnome_vfs_uri_to_string (priv->uri, GNOME_VFS_URI_HIDE_NONE));
-#endif
-
-	g_assert (priv->image == NULL);
-
-	priv->bytes = eog_image_determine_file_bytes (img, error);
-	if (priv->bytes == 0 && (error == NULL || *error != NULL)) {
-		return FALSE;
-	}
-
-	result = gnome_vfs_open_uri (&handle, priv->uri, GNOME_VFS_OPEN_READ);
-	if (result != GNOME_VFS_OK) {
-		g_set_error (error, EOG_IMAGE_ERROR, EOG_IMAGE_ERROR_VFS,
-			     gnome_vfs_result_to_string (result));
-		return FALSE;
-	}
-	
-	buffer = g_new0 (guchar, READ_BUFFER_SIZE);
-	failed = FALSE;
-	bytes_read_total = 0;
-
-	while (!priv->cancel_loading && (first_run || !eog_metadata_reader_finished (md_reader))) {
-
-		result = gnome_vfs_read (handle, buffer, READ_BUFFER_SIZE, &bytes_read);
-		if (result == GNOME_VFS_ERROR_EOF || bytes_read == 0) {
-			break;
-		}
-		else if (result != GNOME_VFS_OK) {
-			failed = TRUE;
-			g_set_error (error, EOG_IMAGE_ERROR, EOG_IMAGE_ERROR_VFS,
-				     gnome_vfs_result_to_string (result));
-			break;
-		}
-		
-		bytes_read_total += bytes_read;
-
-		if (job != NULL) {
-			float progress = (float) bytes_read_total / (float) priv->bytes;
-			eog_job_set_progress (job, progress);
-		}
-		
-		/* check if we support reading metadata for that image format (only JPG atm) */
-		if (first_run) {
-			md_reader = check_for_metadata_img_format (img, buffer, bytes_read);
-			g_set_error (error, EOG_IMAGE_ERROR, EOG_IMAGE_ERROR_GENERIC,
-				     _("EXIF not supported for this file format."));
-			first_run = FALSE;
-		}
-		
-		if (md_reader != NULL) {
-			eog_metadata_reader_consume (md_reader, buffer, bytes_read);
-		}
-	}
-
-	g_free (buffer);
-	gnome_vfs_close (handle);
-	
-	failed = (failed || bytes_read_total == 0 || md_reader == NULL || priv->cancel_loading);
-
-	if (priv->cancel_loading) {
-		priv->cancel_loading = FALSE;
-		priv->status = EOG_IMAGE_STATUS_UNKNOWN;
-	}
-	else if (!failed && (md_reader != NULL)) {
-		/* update meta data */
-#ifdef HAVE_EXIF  
-		priv->exif = eog_metadata_reader_get_exif_data (md_reader);
-		priv->exif_chunk = NULL;
-		priv->exif_chunk_len = 0;
-#else
-		eog_metadata_reader_get_exif_chunk (md_reader, &priv->exif_chunk, &priv->exif_chunk_len);
-#endif
-	}
-	
-	/* clean up */
-	if (md_reader != NULL) {
-		g_object_unref (md_reader);
-	}	
-
-	return !failed;
-}
-
 #ifdef HAVE_LCMS
 void
 eog_image_apply_display_profile (EogImage *img, cmsHPROFILE screen)
@@ -738,7 +628,7 @@ static gboolean
 eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error)
 {
 	EogImagePrivate *priv;
-	GdkPixbufLoader *loader;
+	GdkPixbufLoader *loader = NULL;
 	GnomeVFSHandle *handle;
 	guchar *buffer;
 	GnomeVFSFileSize bytes_read;
@@ -748,6 +638,7 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 	gboolean first_run = TRUE;
 	EogMetadataReader *md_reader = NULL;
 	GdkPixbufFormat *format;
+	gboolean read_image_data = (data2read & EOG_IMAGE_DATA_IMAGE);
 
 	g_assert (error == NULL || *error == NULL);
 
@@ -759,7 +650,7 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 
 	g_assert (priv->image == NULL);
 
-	if (priv->file_type != NULL) {
+	if (read_image_data && priv->file_type != NULL) {
 		g_free (priv->file_type);
 		priv->file_type = NULL;
 	}
@@ -777,11 +668,13 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 	}
 	
 	buffer = g_new0 (guchar, READ_BUFFER_SIZE);
-	loader = gdk_pixbuf_loader_new ();
 	failed = FALSE;
 	bytes_read_total = 0;
 
-	g_signal_connect_object (G_OBJECT (loader), "size-prepared", (GCallback) load_size_prepared, img, 0);
+	if (read_image_data) {
+		loader = gdk_pixbuf_loader_new ();
+		g_signal_connect_object (G_OBJECT (loader), "size-prepared", (GCallback) load_size_prepared, img, 0);
+        }
 
 	while (!priv->cancel_loading) {
 
@@ -796,7 +689,7 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 			break;
 		}
 		
-		if (!gdk_pixbuf_loader_write (loader, buffer, bytes_read, error)) {
+		if (read_image_data && !gdk_pixbuf_loader_write (loader, buffer, bytes_read, error)) {
 			failed = TRUE;
 			break;
 		}
@@ -811,21 +704,31 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 		/* check if we support reading metadata for that image format (only JPG atm) */
 		if (first_run) {
 			md_reader = check_for_metadata_img_format (img, buffer, bytes_read);
+			if (data2read == EOG_IMAGE_DATA_EXIF && md_reader == NULL) {
+				g_set_error (error, EOG_IMAGE_ERROR, 
+                                             EOG_IMAGE_ERROR_GENERIC,
+                                             _("EXIF not supported for this file format."));
+				break;
+                        }
 			first_run = FALSE;
 		}
 		
 		if (md_reader != NULL) {
 			eog_metadata_reader_consume (md_reader, buffer, bytes_read);
+			if (data2read == EOG_IMAGE_DATA_EXIF && eog_metadata_reader_finished (md_reader))
+				break;
 		}
 	}
 
-	/* if we already failed ignore errors on close */
-	if (failed) {
-		gdk_pixbuf_loader_close (loader, NULL);
-	}
-	else if (!gdk_pixbuf_loader_close (loader, error)) {
-		failed = TRUE;
-	}
+	if (read_image_data) {
+		/* if we already failed ignore errors on close */
+		if (failed) {
+			gdk_pixbuf_loader_close (loader, NULL);
+		}
+		else if (!gdk_pixbuf_loader_close (loader, error)) {
+			failed = TRUE;
+		}
+        }
 
 	g_free (buffer);
 	gnome_vfs_close (handle);
@@ -844,17 +747,24 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 	}
 	else if (!failed) {
 
-		/* update image data */
-		if (priv->image != NULL) {
-			g_object_unref (priv->image);
+		if (read_image_data) {
+			/* update image data */
+			if (priv->image != NULL) {
+				g_object_unref (priv->image);
+			}
+
+			priv->image = gdk_pixbuf_loader_get_pixbuf (loader);
+			g_assert (priv->image != NULL);
+			g_object_ref (priv->image);		
+			priv->width = gdk_pixbuf_get_width (priv->image);
+			priv->height = gdk_pixbuf_get_height (priv->image);
+
+			/* update file format */
+			format = gdk_pixbuf_loader_get_format (loader);
+			if (format != NULL) {
+				priv->file_type = gdk_pixbuf_format_get_name (format);
+			}
 		}
-
-		priv->image = gdk_pixbuf_loader_get_pixbuf (loader);
-		g_assert (priv->image != NULL);
-		g_object_ref (priv->image);		
-		priv->width = gdk_pixbuf_get_width (priv->image);
-		priv->height = gdk_pixbuf_get_height (priv->image);
-
 		/* update meta data */
 		if (md_reader != NULL) {
 #ifdef HAVE_EXIF  
@@ -867,11 +777,6 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 #endif
 		}
 
-		/* update file format */
-		format = gdk_pixbuf_loader_get_format (loader);
-		if (format != NULL) {
-			priv->file_type = gdk_pixbuf_format_get_name (format);
-		}
 #ifdef HAVE_LCMS
 		if (md_reader != NULL) {
 			extract_profile (img, md_reader);
@@ -879,7 +784,8 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 #endif
 	}
 	/* clean up */
-	g_object_unref (loader);
+	if (loader != NULL)
+		g_object_unref (loader);
 	if (md_reader != NULL) {
 		g_object_unref (md_reader);
 		md_reader = NULL;
@@ -965,12 +871,7 @@ eog_image_load (EogImage *img, guint data2read, EogJob *job, GError **error)
 	}
 
 	/* Read the requested data from the image */
-	if (data2read == EOG_IMAGE_DATA_EXIF) {
-		success = eog_image_load_exif_data_only (img, job, error);
-	}
-	else {
-		success = eog_image_real_load (img, data2read, job, error);
-	}
+	success = eog_image_real_load (img, data2read, job, error);
 
 #ifdef DEBUG
 	g_print ("load success: %i\n", success);
@@ -1143,7 +1044,7 @@ image_transform (EogImage *img, EogTransform *trans, gboolean is_undo, EogJob *j
 	
 	if (!is_undo) {
 		g_object_ref (trans);
-		priv->undo_stack = g_list_prepend (priv->undo_stack, trans);
+		priv->undo_stack = g_slist_prepend (priv->undo_stack, trans);
 	}
 }
 
@@ -1172,7 +1073,7 @@ eog_image_undo (EogImage *img)
 
 		image_transform (img, inverse, TRUE, NULL);
 
-		priv->undo_stack = g_list_delete_link (priv->undo_stack, priv->undo_stack);
+		priv->undo_stack = g_slist_delete_link (priv->undo_stack, priv->undo_stack);
 		g_object_unref (trans);
 		g_object_unref (inverse);
 
@@ -1296,17 +1197,13 @@ static void
 eog_image_reset_modifications (EogImage *image)
 {
 	EogImagePrivate *priv;
-	GList *it = NULL;
 
 	g_return_if_fail (EOG_IS_IMAGE (image));
 
 	priv = image->priv;
 
-	/* free the undo stack */
-	for (it = priv->undo_stack; it != NULL; it = it->next) {
-		g_object_unref (G_OBJECT (it->data));
-	}
-	g_list_free (priv->undo_stack);
+	g_slist_foreach (priv->undo_stack, (GFunc) g_object_unref, NULL);
+	g_slist_free (priv->undo_stack);
 	priv->undo_stack = NULL;
 
 	/* free accumulated transform object */
@@ -1881,60 +1778,42 @@ eog_image_data_unref (EogImage *img)
 	return img;
 }
 
-/* Print API */
-
-void
-eog_image_print (EogImage *img, GnomePrintContext *context, gdouble paper_width, gdouble paper_height)
-{	
+static gint
+eog_image_get_exif_orientation (EogImage *img)
+{
+	gint orientation = 0;
 	EogImagePrivate *priv = NULL;
-	GdkPixbuf *printed_image = NULL;
-	gint pix_width;
-	gint pix_height;
-	gdouble width, height;
 
-	g_return_if_fail (EOG_IS_IMAGE (img));
-	
+	g_return_val_if_fail (EOG_IS_IMAGE (img), 0);
+
 	priv = img->priv;
-
-	g_return_if_fail (GDK_IS_PIXBUF (priv->image));
-	
-	if (gdk_pixbuf_get_width (priv->image) > gdk_pixbuf_get_height (priv->image)) {
-		printed_image = gdk_pixbuf_rotate_simple (priv->image, 
-			GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
-	} else {
-		g_object_ref (priv->image);
-		printed_image = priv->image;
+	  
+#ifdef HAVE_EXIF
+	if (priv->exif != NULL) {
+		ExifEntry *e = exif_data_get_entry (priv->exif, EXIF_TAG_ORIENTATION);
+		ExifByteOrder o = exif_data_get_byte_order (e->parent->parent);
+		if (e->data != NULL)
+			orientation = exif_get_short (e->data, o);
 	}
+#endif /* HAVE_EXIF */
 
-	pix_width = gdk_pixbuf_get_width (printed_image);
-	pix_height = gdk_pixbuf_get_height (printed_image);
-
-	width = (gint)paper_width;
-	height = (gint)paper_height;
-
-	if (((gdouble) pix_height/pix_width) >
-	    ((gdouble)width/height)) {
-		/* We scale to the top */
-		width = height * (gdouble)pix_width/pix_height;
-	} else {
-		/* We scale to the sides of the page */
-		height = width * (gdouble)pix_height/pix_width;
-	}
-
-	width -= 40;
-	height -= 40;
-	
-	gnome_print_translate (context, (paper_width - width)/2.0, (paper_height - height)/2.0);
-	gnome_print_scale (context, width, height);
-	
-	if (gdk_pixbuf_get_has_alpha (printed_image)) {
-		gnome_print_rgbaimage (context, gdk_pixbuf_get_pixels (printed_image), 
-			pix_width, pix_height, gdk_pixbuf_get_rowstride (printed_image));
-	} else {
-		gnome_print_rgbimage (context, gdk_pixbuf_get_pixels (printed_image), 
-			pix_width, pix_height, gdk_pixbuf_get_rowstride (printed_image));
-	}
-  
-	g_object_unref (G_OBJECT (printed_image));	
+	return orientation;
 }
 
+void
+eog_image_autorotate (EogImage *img)
+{
+	gint orientation = 0;
+
+	g_return_if_fail (EOG_IS_IMAGE (img));
+
+	orientation = eog_image_get_exif_orientation (img);
+
+	EogTransformType type = eog_transform_convert_exif_orientation (orientation);
+
+	if (type != EOG_TRANSFORM_NONE) {
+		EogTransform *transform = eog_transform_new (type);
+		eog_image_transform (img, transform, NULL);
+		g_object_unref (transform);
+	}
+}
